function [ f,z,z0 ] = rowspace_hyperplane_random_combination( n,m,s,A_lds,zidx,k,tol,block_size )
%
% Search for a nontrivial Bell-inequality that is in the orthogonal
% complement of the normalized and zero-constrained non-signaling space.
%
%
% INPUT
% n: number of measurement settings per party
% m: measurement outcomes per setting
% s: number of lds's in the orthogonal complement we try to fit on the hyperplane
% A_lds: local deterministic strategies (all)
% zidx: zero constraint
% k: number of combinations to try
% tol: tolerance for violation of inequality and svd (default: 1e-10)
% block_size: random combinations are generated in blocks (default: k)
%
% OUTPUT
% f: violation of Bell inequality
% (z,z0): affine hyperplane
%

if nargin < 8
    block_size = k;
end

if nargin < 7
    tol = 1e-10;
end

% get NS-basis and NS conditions (with zero contraints)
normalization = true;
[B_zero,C] = ns_basis(n,m,zidx,normalization);

ridx = (sum(A_lds(zidx,:))==0);
A_null = A_lds(:,ridx); % in null space of B
A_row = A_lds(:,~ridx); % in row space of B

% choose any point that is in null space (used to find affine translation
% of hyperplane and calculate B_lds_*)
b = A_null(:,1);

% shift space to make it a vector space (not an affine space)
% B_lds_null = A_null - repmat(b,1,size(A_null,2));
B_lds_row = A_row - repmat(b,1,size(A_row,2));


% number of lds in row space
n_row = size(B_lds_row,2);

% how many possible combinations are there?
% t = nchoosek(n_row,s);

% find a reduced C (C is not of full rank)
[U,S,V] = svd(C);
r = sum(diag(S) > tol); % rank of C
C_red = V(:,1:r)'; % orthonormal basis for row space of C
n_c = r;

% NS conditions minus the zero constraints
C_ns = C(1:end-size(zidx,1),:);
% Basis of ns space (normalized, for is_nontrivial_bell)
B_ns = null(C_ns);

% this usually takes the most time ...
% fprintf(['Creating list of combinations...']);
% if s == 1
%     selections = (1:n_row)';
% else
%     selections = nchoosek(1:n_row,s);
% end
% rperm = randperm(t); % randomly permute order
% disp(['done']);

disp(['Starting search, ', num2str(k),' combinations random will be tried.']);

progressbar(0);
% i = 1;

% generate ranom selections in blocks (just runs faster)
block_position = block_size+1;
block = zeros(block_size,s);

for i = 1:k
    
    if block_position > block_size
        block = randi([1 n_row],block_size,s);
        block_position = 1;
    end
    
    selection = block(block_position,:);
    block_position = block_position + 1;

    
    progressbar(i/k);
    B_lds_max = B_lds_row(:,selection);

    cvx_begin quiet
        variable x(1,n_c);
        maximize sum(x*C_red*B_lds_max);
        subject to
            x*C_red*B_lds_row <= 0;
            x*C_red >= 0;
            x*C_red*b <= 1;
    cvx_end

    x = x';
    z = C_red'*x;
    z0 = z'*b;
%     disp_behaviour(z,n,m);
    f = is_nontrivial_bell(n,m,z,z0,B_ns,b);

    if f > tol
        break;
    end

    i = i + 1;

end

if (f > tol)
    disp(['Found non-trivial Bell. f: ',num2str(f), ' i:', num2str(i)]);
    disp_behaviour(z,n,m);
end
progressbar(1);


end

