function [ fval,z,z0 ] = seperating_hyperplane( p, A, fval_max, Aeq, B )
%
% Find hyperplane between p and behaviors in A
%
% INPUT
% p: ((n*m)^2 X 1) vector of behaviour
% A: ((n*m)^2 x m^2n) matrix with local deterministic strategies in columns
% n: number of possible measurement settings on both sides
% m: number of possible outcomes per measurement
% fval_max: maximum
% Aeq: additional point that should lie on hyperplane (equality
% constraints)
% B: Basis for normal vector. If you want to fit the hyperplane into a
% subspace of R^d you can specify the basis of that subspace here (default:
% eye(d))
%
% OUTPUT:
% fval: distance between point and hyperplane (check for NaN!)
% z: normal of hyperplane
% z0: shift constant
%
% The hyperplane is given as z' * x = z0 for points x on the hyperplane.
%

% size of strategies
d = size(A,1);

if (nargin < 5)
    B = eye(d);
    dB = d;
else
    dB = size(B,2);
end

if (nargin < 4) || isempty(Aeq)
    eqc = false;
    Aeq = [];
%     Aeq = zeros(d+1,1);
else
    eqc = true;
end

if nargin < 3
    fval_max = 1;
end

% number of deterministic strategies
s = size(A,2);
seq = size(Aeq,2);


% set up A for LP
At = A';
Aeqt = Aeq';


% set up right hand side for LP
b = zeros(s,1);
beq = zeros(seq,1);

pt = p';

% lb = zeros(d,1);

if eqc
    % include equality constriants
    cvx_begin quiet
        variables x0 x(dB);
        maximize pt*B*x - x0
        subject to
            (At * B*x) - x0 <= 0;
            Aeqt * B*x - x0 == beq;
            pt * B*x <= fval_max;
            B*x >= 0;
            x0 >= 0;
    cvx_end
else
    cvx_begin quiet
        variables x0 x(dB);
        maximize pt*B*x - x0
        subject to
            (At * B*x) - x0 <= 0;
            pt * B*x <= fval_max;
            B*x >= 0;
            x0 >= 0;
    cvx_end    
end


fval = cvx_optval;

% 
z = B*x;
z0 = x0;


end

