function KL = dist_KL (dm1, dm2)
%
% Compute the Kullback-Leibler "distance" between two matrices.
%
% Usage: d = KL_dist(dm1, dm2)
%     dm1 - The first dm
%     dm2 - The second dm (note: KL is no symetric)
%     d   - The distance
%
% Notes: 
% 1.  The KL "distance" is not a true distance as it neither symmetric
%     not does it satisfy the triangle inequality (d(A,B)+d(B,C) >= d(A,C)).
% 2.  KL is zero for identical states and +Inf for ortogonal ones
%
% Notes regarding the method of computation:
%    The naive implementation is 
%        d = trace(dm1 * (log_dm(dm1) - log_dm(dm2)));
%    Unfortunately, this does not handle infinites nicely.
%    Trivially, we break this down to -trace(d1 log_dm(d2)) - S_Von_Neumann(d1)
%    We then compure the trace is the dm2-diagonal basis, v2: d2 = v2' * dm2 * v2
%    We can then relace the trace with: sum( (v2'*dm1*v2)_i,i * log2(d2_i,i) )
%    This allows us to handle the infinites as for dist_KL_CPD:
%        0 log 0 = 0     (as per usual with entropy)
%        p log 0 = +Inf  (for p > 0)
%


[v1,d1_mat] = eig(dm1); d1 = diag(d1_mat);
[v2,d2_mat] = eig(dm2); d2 = diag(d2_mat);

d_dm1_in_dm2_base = diag(v2' * dm1 * v2); % All nice: non Infs of NaNs

z1 = logical(d_dm1_in_dm2_base == 0); 
z2 = logical(d2 == 0);
if any(z2 & ~z1)
    KL = +Inf;
else
    z12 = ~(z1 | z2);
    
    KL = - sum(d_dm1_in_dm2_base(z12).*log2(d2(z12)));
    KL = KL - H_Shannon(d1);
end

KL = real(KL); % Numerics may cause tiny imaginary part