function [participation, generators_struct_out] = exp_rep_for_of_unitary_op (unitary_op, desc, generators_struct_in)
%
% Compute the exponential representation of a unitary operator: U = expm (i*H)
% with H being a sum kron-ed single-particle SU(desc(k)) generators
%
% Usage: [participation<, generators_struct_out>] = ...
%             exp_rep_for_of_unitary_op (unitary_op<, desc<, generators_struct_in>>)
%     unitary_op                The operator to be analyzed
%     desc                      The DoF descriptor for the operator (optional for all qbits)
%     generators_struct_in      The output of prod_SU_gens for this descriptor (optional)
%     participation             The participation (coeficient) for each generator in generators_struct_in
%     generators_struct_out     The output of prod_SU_gens for this descriptor
%
% As computing the generators_struct, using prod_SU_gens, may be a little
% time-consuming, the function will use this parameter if provided or
% generate it if not.
%
% See demo_log_mat_and_prod_gens_for_exponent_decomp for usage example.
%

if nargin < 2
    desc = gen_desc(unitary_op);
end

if nargin < 3 
    generators_struct_in = prod_SU_gens(desc);
end

H = cleanup(logm (unitary_op)) / i;
participation = NaNs(size(generators_struct_in.which_gen,1),1);
for k = 1:size(generators_struct_in.which_gen,1)
    g = generators_struct_in.gen{k};
    participation(k) = cleanup(trace(H' * g) / trace(g'*g));
end

if nargout > 1
    generators_struct_out = generators_struct_in;
end


