function [new_pure, new_desc] = purify_compact (dm, desc)
%
% Purify a density matrix, adding a single ancilla of the minimum required DoF
%
% Usage: [new_pure, new_desc] = purify_compact (dm<, desc>)
%     dm          The density matrix
%     desc        The DM's descriptor. Optional if all qbits
%     new_pure    The purified state
%     new_desc    The descriptor of new_pure
%
% purify_compact adds a single ancilla particle to the system, with the  
% ancilla's DoF being equal to the number of the DM's non-zero eigenvalues.
% The resulting descriptor is therefore longer by one, and new_desc(end) is
% the ancilla's DoF.
%
% See also: purify_full
%


global QLib;

if nargin < 2
    desc = gen_desc(dm);
end
desc = to_row(desc);
prod_desc = prod(desc);

[v,d] = eig(dm); d = cleanup(diag(d));

N = sum(abs(d) > QLib.close_enough);

new_sz = prod_desc * N;
new_pure = zeros(new_sz,1);

n = 0;
for k=1:length(d)
    if abs(d(k)) > QLib.close_enough
        n = n + 1;
        psi_i = v(:,k);
        p_i = sqrt(d(k));
        i_pure = zeros(N,1); i_pure(n) = 1;
        new_pure = new_pure + p_i * kron(psi_i, i_pure);
    end
end

% Why? Cause I'm treating the dm's DoF as a single DoF when kron-ing,
% we need to re-adjust
new_pure = reorder_and_merge_pure_dims(new_pure, [length(desc):-1:1 (length(desc)+1)], [fliplr(desc) N]);

new_desc = [desc N];
