function [new_pure, new_desc] = purify_full (dm, desc)
%
% Purify a density matrix, by creating ancillas with the same desc as the
% original system.
%
% Usage: [new_pure<, new_desc>] = purify_full (dm<, desc>)
%     dm          The density matrix
%     desc        The DM's descriptor. Optional if all qbits
%     new_pure    The purified state
%     new_desc    The descriptor of new_pure, optional
%
% purify_full adds multiple ancillas. For example, if the system's desc was
% [3 2 5], then the ancillas will also be [3 2 5] with a final new_pure 
% descriptor of [3 2 5 3 2 5]. To contrast, purify_compact would create a
% pure state with a descriptor "between" [3 2 5 1] and [3 2 5 30] (since 30=3*2*5)
%
% See also: purify_compact
%

global QLib;

if nargin < 2
    desc = gen_desc(dm);
end
desc = to_row(desc);
prod_desc = prod(desc);

[v,d] = eig(dm); d = cleanup(diag(d));

new_sz = prod_desc^2;
new_pure = zeros(new_sz,1);

for k=1:length(d)
    psi_i = v(:,k);
    p_i = sqrt(d(k));
    i_pure = zeros(prod_desc,1); i_pure(k) = 1;
    new_pure = new_pure + p_i * kron(psi_i, i_pure);
end

% Why? Cause I'm treating the dm's DoF as a single DoF when kron-ing,
% we need to re-adjust
new_pure = reorder_and_merge_pure_dims(new_pure, ...
    [length(desc):-1:1 length(desc)+(length(desc):-1:1)], [fliplr(desc) fliplr(desc)]);

if nargout > 1
    new_desc = [desc desc];
end
