function yes_no_nan = is_entangled_pt(state, bipartition_mask, desc)
%
% Tries to ascertain if a given state is entangled using the Peres-Horodecki criteria.
%
% Usage: yes_no_nan = is_entangled_pt(dm<, bipartition_mask<, desc>)
%     
%     state       The density matrix or pure state
%     bipartition_mask        How to do the bipartite division of the state
%     desc        The DM's descriptor (optional for qbits)
%     yes_no_nan  If the dm has a descriptor [2 2], [2 3] or [3 2] the
%                 answer will be either yes (1) or no (0). Otherwise, the answer will be 
%                 either yes (1) or "i don't know" (NaN)
%
% Reference:
%     [1] Separability of Mixed States: Necessary and Sufficient Conditions, Horodecki M, Horodecki P, Horodecki R, quant-ph/9605038
%     [2] Separability criterion for density matrices, A. Peres, quant-ph/9604005
%

if nargin < 3
    desc = gen_desc(state);
end
if nargin < 2
    bipartition_mask = [1 0];
end

if size(state,2) == 1
    state = pure2dm(state);
end

pt_dm = partial_transpose(state, bipartition_mask, desc);
eg_vals = cleanup(eig(pt_dm)); % clean - to avoid leftover imaginary
min_eig = min(eg_vals);   

if min_eig < 0
    yes_no_nan = 1;
elseif length(desc) > 2
    yes_no_nan = NaN;
elseif is_eql(desc,[2 2]) || is_eql(desc,[3 2]) || is_eql(desc,[2 3])
    yes_no_nan = 0;
else
    yes_no_nan = NaN;
end
