function [ret_KL, sep_dm_p] = relative_entanglement(dm, desc, varargin)
%
% Compute the relative entanglement: the distance to the closest separable
% state (as measured by dist_KL).
%
% Usage: [ret_KL, sep_dm_p] = relative_entanglement(dm, desc, varargin)
%     dm       The density matrix who's rel. ent. we want to know
%     desc     The dm's descriptor
%     varargin Additional parameters for the fminunc call
%     ret_KL   The relative entropy
%     sep_dm_p The parameter for param_sep_2x which yielded the closest separable state
%
% Notes:
% 1.  Relative entanglement is found numerically by searching the space of
%     separable states for the one giving the minimal dist_KL. 
% 2.  This search may take a long time, depending on the descriptor. 
% 3.  To see progress , add  ...'Display','iter'... to the parameter list.
%

global QLib;

    function KL = search_func (p)
        sep_dm = param_sep_2x(p, desc);
        KL = dist_KL(dm, sep_dm);
    end

if nargin == 1
    desc = gen_desc(dm);
end

sz = param_sep_2x_size (desc);

% [ret_p, ret_KL] = fmin_combo(@search_func, randrow(sz)*1000, ...
%                          fmin_combo_opt('print',0, 'fmin_anneal', fmin_anneal_opt('max_consec_rejections',100, 'max_try', 100), ...
%                                         varargin{:}));

[ret_p, ret_KL] = fminunc (@search_func, rand(1,sz)*1000, ...
                              optimset('LargeScale', 'off', ...
                                       'Display','off',...
                                       'MaxFunEvals', 1e7, ...
                                       'MaxIter', 1e6, ...
                                       'TolFun', QLib.close_enough, ...
                                       'TolX', QLib.close_enough, ...
                                       varargin{:}));
        % Seprable state parametrization has the unfortunate attribute that for some values (such as lots of 
        % leading zeros) the state ends up as all NaNs. Therefore we start with a
        % random param-point and not all zeros


if nargout > 1
    sep_dm_p = ret_p;
end

end
