function [sf, maximazing_singlet] = singlet_fraction (dm, bipartition_mask, desc)
%
% Compute (by searching for) the (generalized) singlet fraction for any given bipartition of any dm
%
% Usage: [sf, maximazing_singlet] = singlet_fraction (dm, bipartition_mask, desc)
%     dm                   The density matrix (any dimentionality)
%     bipartition_mask     Not needed for two qbits
%     desc                 The DM's descriptor. Optional for qbits
%     sf                   The singlet fraction
%     maximazing_singlet   The psi (see below) which maximizes the fraction
%
% The singlet fraction is the value of <psi | DM | psi>, maximized over all
% rotations of the singlet psi
%
% Reference:
% [1] "General teleportation channel, singlet fraction and quasidistillation", the M, P & R Horodecki, Phys Rev A, 60(3), p 1888 (1999)
%

if nargin < 3
    desc = gen_desc(dm);
end

if (nargin < 2) && (length(desc) == 2)
    bipartition_mask = [1 0];
end

bimask = logical(bipartition_mask);

if prod(desc(bimask)) ~= prod(desc(~bimask))
    error ('Bipartition must be symmetric');
end

n = prod(desc(bimask));
base_singlet = GHZ(n);

    function f = fraction (U_param)
        U = param_U_2(U_param);
        rotated_singlet = U * base_singlet;
        f = real (rotated_singlet' * dm * rotated_singlet); % "real" to get rid of tiny imaginary parts
    end

search_options = fmin_combo_opt ('print', -1);

[best_p, best_val] = fmax_combo(@(p) fraction(p), 1000*randrow(param_U_2_size(desc)), search_options);

sf = best_val;

if nargout > 1
    maximazing_singlet = param_U_2(best_p) * base_singlet;
end

end