function t = tangle (dm_22_or_pure_222)
%
% Compute the tangle, as follows:
%     For a 3 qbit pure state, this is the three-tangle as defined by Coffman, Kundu and Wootters 
%     For a 2 qbit DM, the tangle is identical to the concurrence squared
%
% Usage: t = tangle (dm_22_or_pure_222)
%     dm_22_or_pure_222   A 3 qbit pure state or 2 qbit DM
%     t                   The resulting tangle
%
% Reference: 
% [1] "Distributed entanglement" Coffman, Kundu and Wootters, Phys. Rev. A 61, 052306 (2000) eq. 17 & 24
%     

if (size(dm_22_or_pure_222,1) == 4) && (size(dm_22_or_pure_222,2) == 4)
    % 2-qbit dm
    t = concurrence(dm_22_or_pure_222)^2;
    return;
end

% Assuming 3-qbit pure state

sigma22 = [0 0 0 -1; 0 0 1 0; 0 1 0 0; -1 0 0 0];


ab = partial_trace_pure(dm_22_or_pure_222, [1 1 0], [2 2 2]);
ac = partial_trace_pure(dm_22_or_pure_222, [0 1 1], [2 2 2]);

ab_tilde = sigma22 * conj(ab) * sigma22;
ac_tilde = sigma22 * conj(ac) * sigma22;

ab_e = sort(eig(ab*ab_tilde)).^(1/2);
ac_e = sort(eig(ac*ac_tilde)).^(1/2);

t = real(2*(ab_e(3)*ab_e(4) + ac_e(3)*ac_e(4))); % Remove tiny imaginary part

%%
%
% An alternate implementation, as per eq. 
%

% global QLib; e = QLib.eps{2};
% 
% tnsr = pure2tensor(pure_222, [2 2 2]);
% sm = 0;
% 
% for i1=1:2
%     for i2=1:2
%         for j1=1:2
%             for j2=1:2
%                 for k1=1:2
%                     for k2=1:2
%                         for m1=1:2
%                             for m2=1:2
%                                 for n1=1:2
%                                     for n2=1:2
%                                         for p1=1:2
%                                             for p2=1:2
%                                                 sm = sm + e(i1,i2) * e(j1,j2) * e(k1,k2) * e(m1,m2) * e(n1,n2) * e(p1,p2) * ...
%                                                             tnsr(i1,j1,k1)*tnsr(i2,j2,m1)*tnsr(n1,p1,k2)*tnsr(n2,p2,m2);
%                                             end
%                                         end
%                                     end
%                                 end
%                             end
%                         end
%                     end
%                 end
%             end
%         end
%     end
% end
% 
% t = 2*abs(sm);

