function [ret_KL, prod_dm_p] = relative_entropy(dm, desc, varargin)
%
% Compute the relative entropy: the distance to the closest product
% state (as measured by dist_KL).
%
% Usage: [ret_KL, prod_dm_p] = relative_entropy(dm, desc, varargin)
%     dm        The density matrix who's rel. ent. we want to know
%     desc      The dm's descriptor
%     varargin  Additional parameters for the fminunc call
%     ret_KL    The relative entropy
%     prod_dm_p The parameter for param_prod which yielded the closest separable state
%
% Notes:
% 1.  Relative entropy is found numerically by searching the space of
%     product states for the one giving the minimal dist_KL. 
% 2.  This search may take a long time (but less than relative_entanglement)
% 3.  To see progress , add  ...'Display','iter'... to the parameter list.
%


global QLib;

    function KL = search_func (p)
        prod_dm = param_prod(p, desc);
        KL = dist_KL(dm, prod_dm);
    end

if nargin == 1
    desc = gen_desc(dm);
end

sz = sum(desc.^2)-length(desc);


% [ret_p, ret_KL] = fmin_combo(@search_func, randrow(sz)*1000, ...
%                          fmin_combo_opt('print',0, 'fmin_anneal', fmin_anneal_opt('max_consec_rejections',100, 'max_try', 100), ...
%                                         varargin{:}));
                                    
[ret_p, ret_KL] = fminunc (@search_func, rand(1,sz)*100, ...
                              optimset('LargeScale', 'off', ...
                                       'Display','off',...
                                       'MaxFunEvals', 1e7, ...
                                       'MaxIter', 1e6, ...
                                       'TolFun', QLib.close_enough, ...
                                       'TolX', QLib.close_enough, ...
                                       varargin{:}));
        % Product state parametrization has the unfortunate attribute that for some values (such as lots of 
        % leading zeros) the state ends up as all NaNs. Therefore we start with a
        % random param-point and not all zeros


if nargout > 1
    prod_dm_p = ret_p;
end

end