function ret = param_SU_over_U(p)
% 
% Parametrization of SU(N)/U(1)^N
%
% Usage: ret = param_SU(p)
%     p - Vector of N(N-1) elements
%     ret - The SU(N)/U(1)^N member
%
% U(N), in it's role as a matrix diagonalizer (as in H = U^-1*D*U), has a
% nasty redundency: Each column (eigenvector) may be multiplied by a phase.
% If we were to use U(N) to parametriz H (or a dm) as above, we would end
% up requiring N(N+1) parameters, which is N too much.
% To get rid of these parameters, see references below, specifically section 4 of [1].
%
% Notes:
%     * The routine determines the N in SU(N) from the length of p
%     * Although all parameters are cyclic, they have different periods, due to the different eigenvalues 
%       of the generators of the algebra. Order of magnitude is 1/sqrt(N)
%
% References: 
% [1] math-ph/0202002: A Parametrization of Bipartite Systems Based on SU(4) Euler Angles, Tilma, Byrd & Sudarshan
% [2] math-ph/0205016: Generalized Euler Angle Parametrization for SU(N), Tilma & Sudarshan
%

global QLib;

N = (sqrt(4*length(p)+1)+1)*0.5;
ret = eye(N);

if N > length(QLib.SU.param_form)
    SU_gen_cache(N);
end

for k=1:length(p)
    kk = QLib.SU.param_form{N}(k);
    ret = ret * (QLib.SU.gen_eigvec{N}{kk}*diag(exp(i*QLib.SU.gen_eigval{N}{kk}*p(k)))*QLib.SU.gen_inv_eigvec{N}{kk});
    % Same form as general SU(N) parametrization, except we do not use the
    % last N-1 elements.
end
