function varargout = eig2(a)
%
% eig2: A modified version of Matlab's "eig":
%     Eigenvectors & eigenvalues are sorted
%     Eigenvectors are normalized (v'*v == 1)
%     Explicit support for left and right eigenvectors for non-normal matrices
%
% Uses:
%     d = eig2(a)                      Just the eigenvalues
%     [e,d] = eig2(a)                  For normal matrices
%     [left_e, right_e, d] = eig2(a)   For non-normal matrices
%
% Notes:
% 1.  The following code will reconstruct the matrix (when non singular)
%     s = zeros(N);
%     for k=1:N
%         left_v  = left_e (:,k);
%         right_v = right_e(:,k);
%         normalization = transpose(left_v)*right_v;
%         if ~is_close(normalization,0)
%             z = right_e(:,k)*transpose(left_e(:,k)) / normalization;
%             s = s + d(k) * z;
%         end
%     end
%

global QLib;

[e,d] = eig(a);
e = cleanup(e); d1 = to_row(cleanup(diag(d)));
[tmp,I] = sort(d1);

d = diag(d1(I));

if nargout == 1
    varargout{1} = d;
    return;
end

if nargout == 2
    if ~is_normal_matrix(a)
        error ('Eigenvectors not well defined for non-normal matrices');
    end
    ee = NaNs(size(e));
    for k=1:length(ee)
        v = e(:,I(k));
        v = v ./ (v'*v);
        ee(:,k) = v;
    end
    varargout{1} = ee;
    varargout{2} = d;
end

if nargout == 3
    ee = NaNs(size(e));
    for k=1:length(ee)
        v = e(:,I(k));
        v_norm = v'*v;
        if v_norm > QLib.close_enough
            ee(:,k) = v/v_norm;
        else
            ee(:,k) = NaN;
        end
    end
    varargout{2} = ee; % Right eigenvalues
    varargout{3} = d;

    [e,d] = eig(transpose(a));
    e = cleanup(e); d1 = to_row(cleanup(diag(d)));
    [tmp,I] = sort(d1);
    ee = NaNs(size(e));
    for k=1:length(ee)
        v = e(:,I(k));
        v_norm = v'*v;
        if v_norm > QLib.close_enough
            ee(:,k) = v/v_norm;
        else
            ee(:,k) = NaN;
        end
    end
    varargout{1} = ee; % Left eigenvalues

    return;
end

error ('Too many output arguments');
