function factors = find_span (what, varargin)
%
% find_span: Decompose an object as linear sum of objects of same dimentionality
%
% Usage: factors = find_span (what, varargin)
%     what     The object to be decomposed
%     varargin The set of objects which (hopefully) span "what". We'll call
%              them eigenX. They can be specified as a cell array, as list of
%              parameters, or in case what is a vector, the set can be
%              specified as a matrix.
%     factor   The weights. Length is by number of eigenX
%
% Notes:
%
% 1.  If the set in "varargin" is not mutually orthogonal, the resulting span is
%     dependent on the order in which the "vectors" are supplied.
%     (consider using gram_schmidt first)
% 2.  If the "what" is not in the space spanned by the vectors,
%     the result will be all NaNs
% 3.  This function works not only for vectors, but also for spanning
%     matrices by matrices, tensors by tensors, etc.
% 4.  find_span also supports complex objects
% 5.  There is no need that the eigenX span the entire space for objects of
%     size size(what). You can restrict yourself to any subspace.
% 
% Example: Let us imagin the objects comprising what are in the cell array "eigenXs":
%
%         factors = find_span(what, eigenXs);
%         % Now reconstruct "what"
%         reconstructed_what = zeros(size(what));
%         for k=1:length(eigenXs)
%             reconstructed_what = reconstructed_what + factors(k)*eigenXs{k};
%         end
%         % Alternative reconstruction:
%         reconstructed_what2 = span(factors, eigenX);
%
% See also: span
%

if (nargin == 2) && iscell(varargin{1})
    varargin = varargin{:};
end

m = numel(what);
if isequal(size(what), [1 1])
    % Scalars - turn the arguments into column vectors
    z = {};
    for k=1:length(varargin)
        z{k} = to_col(varargin{k});
    end
    factors = find_span(what, z{:});
    return;
elseif isequal(size(what), [m 1])
    % Column vectors - a simple horizcat
    A = varargin{1};
    for k=2:length(varargin)
        sz = size(varargin{k});
        if isequal(sz,[m 1])
            A(:,end+1) = varargin{k};
        elseif sz(1) == m
            A(:,(end+1):(end+sz(2))) = varargin{k};
        else
            error ('Size of parameter mismatch');
        end
    end
else
    % Scalar, row vector or higher order
    sz_what = size(what);
    if isequal(sz_what, [1 m])
        sz_what = m;
    end
    
    A = [];
    for k=1:length(varargin)
        sz = size(varargin{k});
        if isequal(sz,sz_what)
            A(:,end+1) = to_col(varargin{k});
        elseif isequal(sz(2:end), sz_what)
            A(:,(end+1):(end+sz(1))) = reshape(varargin{k}, [numel(what), sz(1)]);
        else
            error ('Size of parameter mismatch');
        end
    end
end

what = to_col(what);

warning off MATLAB:singularMatrix
warning off MATLAB:nearlySingularMatrix

factors = A\what; % "Backslash or matrix left division. If A is a square matrix, A\B is roughly the same as inv(A)*B, except it is computed in a different way"

warning on MATLAB:nearlySingularMatrix
warning on MATLAB:singularMatrix

if ~is_close(A*factors, what)
    factors = NaNs(1,size(factors));
end