function scale = mat_scale(A,B, exact)
%
% Is there / what is the scale relation between the two matrices?
% In other words, is there a scalar p such that A = pB ?
%
% Usage: scale = mat_scale(A,B <, exact>)
%     A,B       Find the scale of A/B
%     exact     Logical (true/false), optional, defaults to 0
%               If not exact, A and B and the are cleaned up
%     scale     The scale of the two matrices, or NaN if none exist
%
% Notes:
%     Cases of note:
%         A:     a     0  Inf NaN
%       -------+-----------------
%       B:   b | a/b  NaN (2) NaN
%            0 |  0   (1) NaN NaN
%          Inf | NaN  NaN (3) NaN
%          NaN | NaN  NaN NaN NaN
%
%     (1a) If A,B are both all zero, 0 is returned
%     (1b) If the are non (0,0) elements, they determine the ratio.
%          But if the ratio they determine is +- Inf, the final result is NaN, as 0*Inf = NaN
%     (2) Ratio is +-Inf.
%     (3) Ratio is any number, with appropriate sign
%
% Algorithm: Scale can be either a number, 0, +Inf, -Inf or NaN.
%     It's a number iff we can find two "normal" elements and their ratio fits everything.
%     Otherwise, simply test the other three options.
%
% Caution: It is not recommended to work in "exact" mode, unless the
%     matrix is all integers and the ratios have a nice binary
%     representation
%

scale = NaN;
if ~isequal(size(A),size(B))
    return;
end

if nargin == 2
    exact = 0;
end

if ~exact
    A = cleanup(A);
    B = cleanup(B);
end

A_normal = find((A ~= 0) & (~isnan(A)) & (~isinf(A)));
B_normal = find((B ~= 0) & (~isnan(B)) & (~isinf(B)));
if length(intersect(A_normal,B_normal)) > 0
    ratio = A(A_normal(1))/B(A_normal(1));
    if (exact && isequal(B*ratio,A)) || ((~exact) && is_close(B*ratio,A))
        scale = ratio;
        return
    else
        return
    end
end

ratio = 0;
if (exact && isequal(B*ratio,A)) || ((~exact) && is_close_2(B*ratio,A))
    scale = ratio;
    return
end

ratio = +Inf;
if (exact && isequal(B*ratio,A)) || ((~exact) && is_close_2(B*ratio,A))
    scale = ratio;
    return
end

ratio = -Inf;
if (exact && isequal(B*ratio,A)) || ((~exact) && is_close_2(B*ratio,A))
    scale = ratio;
    return
end

% scale is already NaN - nothing to do here
