function what = span (factors, varargin)
%
% span: Reconstructs a decompose an object as linear sum of objects of same dimentionality
%
% Usage: what = span (factors, varargin)
%     factor   The weights for the base elements. See find_span
%     varargin The set of objects which (hopefully) span "what". We'll call
%              them eigenX. They can be specified as a cell array, as list of
%              parameters, or in case what is a vector, the set can be
%              specified as a matrix.
%     what     The object to be decomposed
%
% Notes:
%
% 1.  This function works not only for vectors, but also for spanning
%     matrices by matrices, tensors by tensors, etc.
% 2.  There is no need that the eigenX span the entire space for objects of
%     size size(what). You can restrict yourself to any subspace.
% 
% Example: Let us imagin the objects comprising what are in the cell array "eigenXs":
%
%         factors = find_span(what, eigenXs);
%         % Now reconstruct "what"
%         reconstructed_what = zeros(size(what));
%         for k=1:length(eigenXs)
%             reconstructed_what = reconstructed_what + factors(k)*eigenXs{k};
%         end
%         % Alternative reconstruction:
%         reconstructed_what2 = span(factors, eigenX);
%
% See also: find_span
%

if (nargin == 2) && iscell(varargin{1})
    varargin = varargin{:};
end

if length(varargin) == 1
    d = varargin{1};
    sz = size(d);
    if length(sz) == 2
        % Make sure it's not a single row/column
        if (sz(1) == 1) || (sz(2) == 1)
            what = to_row(factors)*to_col(d);
            return;
        end
        % It's a matrix, assume columns
        varargin = {};
        for k=1:sz(2)
            varargin{k} = d(:,k);
        end
    else
        varargin = {};
        d = reshape(d, [sz(1), prod(sz(2:end))]);
        for k=1:sz(1)
            varargin{k} = reshape(d(k,:), sz(2:end));
        end
    end
end
    
what = zeros(size(varargin{1}));
for k=1:length(factors)
    what = what + factors(k) * varargin{k};
end