function mat = sun2mat (sun)
%
% Convert a matrix (DM, unitary or Hermitian) from the kron(SU(desc(1)),
% SU(desc(2)), ...) basis to the normal representation
%
% For example, the CNOT gate can be written as 1/2 * (-z*x + z*I + I*Z + I*I), 
% where x and z are the Pauli matrices, * is kron and I is eye(2). See below for an example of use.
%
% Usage: mat = sun2mat(sun)
%     sun   The matrix represented in the SU(desc(1))*SU(desc(2))*... basis
%           The number of dimentions in sun is identical to the length of
%           the descriptor (and number of particles). For example, in a 3-qubit system, the
%           coeficient for x*z*y will at sun(1,3,2);
%     mat   The matrix in "normal" form
%  
% Example:
%     QLib.gates.CNOT
%     ans =
%          1     0     0     0                    % Note: First qubit is the control
%          0     1     0     0                    
%          0     0     0     1
%          0     0     1     0                    
%     sun = mat2sun(QLib.gates.CNOT)              % The order of generators for SU(2) is x,y,z,I
%     sun =
%              0         0         0         0    % The first index (row) is the first particle
%              0         0         0         0    % The second index (row) is the second particle
%        -0.5000         0         0    0.5000    % So element at (3,1) indicates the -0.5 z*x coeficient
%         0.5000         0         0    0.5000    % The element at (4,4) indicatez the 0.5 I*I coeficient
%     sun2mat(sun)                  
%     ans =
%          1     0     0     0                    % Note: First qubit is the control
%          0     1     0     0                    
%          0     0     0     1
%          0     0     1     0                    
%
%     % Also see demo_factoring_of_dm 
%
% Notes:
% 1.  The ordering of the SU(N) generators is viewable in QLib.SU.gen{N}
% 2.  Perform SU_gen_cache(desc) if some SU generators are not defined.
% 2.  The function generates and caches all the generator combinations
%     using prod_SU_gens. See there for details
% 3.  This is the inverse of mat2sun
%




desc = size(sun).^(1/2);
if (length(desc) == 2) && (desc(1) == 1)
    desc = desc(2); % Fix for single particle
end

prod_gens = prod_SU_gens(desc);

ind = sub3ind(size(sun), prod_gens.which_gen);

mat = zeros(prod(desc),prod(desc));

for k = 1:length(ind)
    mat = mat + sun(ind(k))*prod_gens.gen{k};
end

