function [out_best_min_point, out_best_min_val, out_num_reps] = fmin_combo (the_func, p2, p3, p4)
%
% A somewhat robust minimization/goal-searching function.
% Utilizes a combination of methods to either attempt and find a function's global
% minima or alternatively find a spot in parameter space where the function
% reaches a requested value.
%
% Usage 1: [best_min_point, best_min_val, num_reps] = fmin_combo (the_func, initial_guess, options)
% Usage 2: [best_min_point, best_min_val, num_reps] = fmin_combo (the_func, goal, initial_guess, options)
%     the_func - The function to minimize
%     initial_guess - As named, but also used to define the dimentionality of the_func's parameters
%     options - Options for both this, the combined search, and for each of the
%         subordiate searches. fmin_combo_opt for further details.
%     best_min_point - The value fed into the_func which produces the minimal value
%     best_min_val - The result of the line above
%     num_reps - How many repititions were required. Negative if error
%         -1    Timeout (CPU)
%         -2    Timeout (elapsed)
%         -3    Itteration limit reached
%
% Important notes:
%     0. There is no way to ascertain if the global minima has been reached
%         or ig we are "stuck" in a local minima. By enabling the
%         warn_on_non_convergence
%     1. Scale: It is assumed that all parameters are of a scale of 1
%        (meaning the minima we're looking for is in (-10..10)^d or there abouts).
%     2. Options: There are 3 distinct sets of parameters you may
%        modify: options.fminunc, options.fmin_anneal and options.fmin_combo
%        For options.fminunc, see fminunc

start_cpu_time = cputime();
start_elapsed_time = clock();

do_goal = 0;

if nargin == 2
    initial_guess = p2;
    options = fmin_combo_opt();
    do_goal = 0;
elseif nargin == 3
    initial_guess = p2;
    options = p3;
    do_goal = 0;
    
elseif nargin == 4
    goal = p2;
    initial_guess = p3;
    options = p4;
    do_goal = 1;
else
    help fmin_combo
    error ('Please specify 2..4 input parameters. "help fmin_combo" for details.');
end

initial_guess = reshape(initial_guess, numel(initial_guess),1);

best_min_val = +Inf;
best_min_point = NaNs(size(initial_guess));
non_improve_count = 0;
consistent_count = 0; consistent_val = NaN;
stepale = options.fmin_combo.initial_step;
num_reps = 0;

for z=1:options.fmin_combo.max_num_steps
    if (cputime() - start_cpu_time) > options.fmin_combo.cpu_time_limit
        num_reps = -1;
        if options.fmin_combo.print >= 0 
            warning ('Search cpu time limit exceeded');
        end
        break;
    end
    if etime(clock(),start_elapsed_time) > options.fmin_combo.elapsed_time_limit
        num_reps = -2;
        if options.fmin_combo.print >= 0 
            warning ('Elapsed time limit exceeded');
        end
        break;
    end
    
    if options.fmin_combo.print == 1
        fprintf('    fmin_combo loop %d ... ', z); drawnow;
    elseif options.fmin_combo.print >= 2
        fprintf('    fmin_combo loop %d\n', z); drawnow;
    end

    if (options.fmin_combo.print >= 2)
        fprintf ('                  anneal: '); drawnow;
    end    
    [min_point, min_val] = fmin_anneal (the_func, ...
                                    @(x) (x + randn/stepale * (randperm(length(x))'==length(x))), ...
                                    initial_guess, 0, options.fmin_anneal);
    if (options.fmin_combo.print >= 2)
        fprintf('%12.10f\n', min_val); drawnow;
    end

    if (options.fmin_combo.print >= 2)
        fprintf ('                  search: '); drawnow;
    end
    [min_point, min_val] = fminunc(the_func, min_point, options.fminunc);
    if (options.fmin_combo.print >= 2)
        fprintf ('%12.10f\n', min_val); drawnow;
    end

    if options.fmin_combo.print == 1
        fprintf ('%g'\n, min_val);
    end

    if do_goal & (abs(min_val-goal) <= options.fmin_combo.y_close_enough) 
        best_min_val = min_val;
        best_min_point = min_point;
        num_reps = z;
        break;
    end
    
    if abs(consistent_val - best_min_val) <= options.fmin_combo.y_close_enough
        consistent_count = consistent_count + 1;
        if consistent_count >= options.fmin_combo.num_consistent_results_to_abort
            num_reps = z;
            break;
        end
    else
        consistent_val = best_min_val;
        consistent_count = 0;
    end
    
    if (best_min_val - min_val) >= options.fmin_combo.y_close_enough
        best_min_val = min_val;
        best_min_point = min_point;
        non_improve_count = 0;
        stepale = stepale / options.fmin_combo.step_shrink_factor;
    else
        non_improve_count = non_improve_count + 1;
        if non_improve_count > options.fmin_combo.num_non_improve_to_abort
            num_reps = z;
            break;
        end
    end
    initial_guess = best_min_point; % If did not improve, don't use new point
end

if (z == options.fmin_combo.max_num_steps)
    num_reps = -3;
    if (options.fmin_combo.print >= 0) && (~options.fmin_combo.no_warnings)
        warning ('Iteration limit reached');
    end
end

if options.fmin_combo.print > 0
    disp (sprintf('            best: %g', best_min_val));
end

out_best_min_point = best_min_point;
if nargout >=2
    out_best_min_val = best_min_val;
end
if nargout >=3
    out_num_reps = num_reps;
end

