function opt = fmin_combo_opt(varargin)
%
% Set options for the fmin_combo search
%
% Usage: opt = fmin_combo_opt(opt_name_1, opt_val_1, opt_name_2, opt_val_2, ...)
%     input: Pairs of option names and their values. See details below
%     output: A structure containing a complete set of values (those set
%         and defaults for the rest).
%
% Option details: The options are divided into 2 categories:
%     Options for the specific searches, currently containing opt.fmin_seach.* and opt.fmin_anneal.*
%         These are set by optimset and fmin_anneal_opt. Details can be found therein.
%     Options for the combination search, residing in opt.fmin_combo.*
%         These are set by this routine and are detailed below:
%
%         initial_step: Size (along all axes) in param-space to search
%         step_shrink_factor: by how much to shrink the param-space every step (> 1)
%
%         print: level of information printed
%             -1: Never print anything. Ever.
%              0: Print warning if optimization stopped on some hard limit (iterations, time, etc)
%                 and not on value limits, such as proximity to goals
%              1: Print basic progress information
%              2: Print detailed progress information.
%
%         max_num_steps: Maximum number of iterations
%         num_consistent_results_to_abort: How many steps of a consistent
%               value (constant up to y_close_enough) do we need to abort?
%         num_non_improve_to_abort: How many steps of non-improvement is enough to terminate?
%         x_close_enough: What is considered "no change" for the condition above
%         y_close_enough: As above, but in the value-space and not the param-space
%         no_warnings: Will not print a warning, regardless of print level
%
%         cpu_time_limit: CPU time limit for the search (up to 1 itteration of included searches)
%         elapsed_time_limit: CPU time limit for the search (up to 1 itteration of included searches)
%
%         fmin_anneal: The output of fmin_anneal_opt
%         fminseach: The output of optimset. This overwrides fmin_combo/fmax_combo default optimset settings
%

global QLib;

% Default values
opt.fmin_combo.initial_step = 100;
opt.fmin_combo.max_num_steps = 99;
opt.fmin_combo.print = 2;
opt.fmin_combo.step_shrink_factor = 1.5;
opt.fmin_combo.num_non_improve_to_abort = +Inf;
opt.fmin_combo.print = 0; % Silent
opt.fmin_combo.x_close_enough = QLib.close_enough;
opt.fmin_combo.y_close_enough = QLib.close_enough;
opt.fmin_combo.cpu_time_limit     = Inf; 
opt.fmin_combo.elapsed_time_limit = Inf;
opt.fmin_combo.no_warnings = 0;
opt.fmin_combo.num_consistent_results_to_abort = 3;
    
opt.fmin_anneal = fmin_anneal_opt();
opt.fminunc  = optimset('LargeScale', 'off', 'Display','off', 'MaxFunEvals', 1e6, 'MaxIter', 1e6);

% Loop on user-supplied values
k = 1;
while k < length(varargin)
    if strcmp(varargin{k},'no_warnings')
        opt.no_warnings = 1;
        k = k + 1;
    elseif strcmp(varargin{k},'fminunc') || strcmp(varargin{k},'fminsearch') 
        opt.fminunc = optimset(opt.fminunc, varargin{k+1});
        k = k + 2;
    elseif isfield(opt,varargin{k})
        opt.(varargin{k}) = varargin{k+1};
        k = k + 2;
    elseif isfield(opt.fmin_combo,varargin{k})
        opt.fmin_combo.(varargin{k}) = varargin{k+1};
        k = k + 2;
    elseif strcmp(varargin{k},'all')
        opt = varargin{k+1};
        k = k + 2;
    else
        disp(' ');
        disp('fmin_combo options (and their default values):');
        disp(opt);
        error ('Unrecognized field name: %s', varargin{k});
    end
end

