%
% This file contains various tests for locality. You will need to define
% certain variables before using this file (e.g. p,n,m), see comment
% in cells.
%

%% Generate the local deterministic strategies, set n,m before using this
disp('Generating local deterministic behaviours')
tic
A_lds = local_deterministic_strategies(n,m);
toc



%% Test if point p is in polytope defined by A_lds
tic
[inp,x] = in_polytope(p,A_lds);


if (inp == 1)
    disp('p is in polytope.');
elseif (inp == 0)
    disp('p is outside of polytope.');
else
    disp('Error: Linear program returned NaN, please investigate.');
end
toc

%% try and fit a hyperplane between p and all A_lds

% if p is outside of polytope, solution will be unbounded unless we set a
% maximal value for the objective function.
f_max = 1;

% fit hyperplane
normalization = false;
[B_ns,C_ns] = ns_basis(n,m,[],normalization);
tic
[fval,z,z0] = seperating_hyperplane(p,A_lds,f_max,[],B_ns);
toc
tic
[fval,z,z0] = seperating_hyperplane(p,A_lds,f_max,[]);
toc

if norm(z) < eps
    warning('Probably trivial hyperplane!');
end

% normalize hyperplane
[z,z0] = normalize_hyperplane(z,z0);

% show pretty graph and print some information, d is a vector containing
% distance to all lds
d = verify_seperating_hyperplane(z,z0,p,A_lds);

%% if point is outside of polytope we might get a hyperplane that does not
 % touch the local deterministic strategies. Here we attempt to find a
 % 'nicer' hyperplane that touches the lds's.

% by how much should the hyperplane be shifted
shift = max(d); % usually what we want to do...
tol = 1e-5;


[x,x0,nop] = face_hyperplane(z,z0,A_lds,tol,shift);
[x,x0] = normalize_hyperplane(x,x0);

disp(['number of points touching hyperplane: ', num2str(nop)]);

d = verify_seperating_hyperplane(x,x0,p,A_lds);

%% find hyperplane in rowspace of C

% number of lds in row space that lie on the hyperplane
s = 2;

tol = 1e-10;
zidx = find(p==0);

% Instead of going throu all combination in random order we may also just
% choose combinations randomly. There is a risk of chooing a combination 
% multiple time and not all combinations may be checked, however it seems 
% to work faster/better. 
%
% We set a maximum number of combinations to check.
k = 1e3;

block_size = k;

% If you want to check all combination systematicly use the first version.
% [f,z,z0] = rowspace_hyperplane(n,m,s,A_lds,zidx,tol);
[f,z,z0] = rowspace_hyperplane_random_combination(n,m,s,A_lds,zidx,k,tol,block_size);

if f > tol
    [z,z0] = normalize_hyperplane(z,z0);
    d = verify_seperating_hyperplane(z,z0,p,A_lds);
end

%% Check chained CHSH inequality

[z,z0] = chsh(n,m,0);

% Note: z,z0 is not normalized
d = verify_seperating_hyperplane(z,z0,p,A_lds);

%% a symmetric opposite chained CHSH
[z,z0] = chsh(n,m,1);
d = verify_seperating_hyperplane(z,z0,p,A_lds);

 
%% Check if hyperplane is a non-trivial Bell-inequality by searching a
 % non-signaling behaviour that violates the inequality.

[f,p_ns] = is_nontrivial_bell(n,m,z,z0);
disp(['non-signaling behaviour violates inequality by: ',num2str(f)]);
disp_behaviour(p_ns,n,m);


