%
% This file contains various interesting measurement setups and generates
% the probabilistic behaviour for a quantum state for these measurement
% setups.
%
% Load a quantum state rho (in wb_states.m) before using this.
%

%% setup n and m

% number of measurement settings per party
n = 2;

% number of outcomes per measurement
m = 2;

%% Measurement in standard basis

M = standard_basis_povm(n,m);

p = q2p(rho,M,M,n,m);
clear('M');

%display p in readable form
disp('The behaviour:');
disp(reshape(p,n*m,n*m));

%% Basis to maximaly violate CHSH (use for Singlet; NOTE: only for n=2!!!)

if(n ~= 2)
    error('Not defined for n~=2');
end

% Pauli matrices
sigma = zeros(2,2,3);
sigma(:,:,1) = [0  1; 1  0];
sigma(:,:,2) = [0 -1i; 1i  0];
sigma(:,:,3) = [1  0; 0 -1];


% set up measurements
d=2;
% measurements on A: Z and X
MA = zeros(d,d,m,n);

MA(:,:,:,1) = observable2povm(sigma(:,:,3));
MA(:,:,:,2) = observable2povm(sigma(:,:,1));


% measurements on B:
MB = zeros(d,d,m,n);

S = (- sigma(:,:,3) - sigma(:,:,1))/sqrt(2);
MB(:,:,:,1) = observable2povm(S);

T = (sigma(:,:,3) - sigma(:,:,1))/sqrt(2);
MB(:,:,:,2) = observable2povm(T);


% compute the quantum behaviour
p = q2p(rho,MA,MB,n,m);

% clean up
clear('MA','MB','d','S','T');

%display p in readable form
disp('The behaviour:');
disp(reshape(p,n*m,n*m));

%% Angles in n steps between alpha max and alpha min (2D)

% for A
alpha0 = 0;
alpha = 1e-3;

% for B
beta0 = 1e-3;
beta = 1e-3;


% standard basis
b = eye(2);


MA = zeros(2,2,m,n);
R = rotation_matrix(2,alpha0);
MA(:,:,1,1) = R * b(:,1) * b(:,1)' * R';
MA(:,:,2,1) = R * b(:,2) * b(:,2)' * R';
R = rotation_matrix(2,alpha);
for i = 2:n
    MA(:,:,1,i) = R * MA(:,:,1,i-1) * R';
    MA(:,:,2,i) = R * MA(:,:,2,i-1) * R';
end

% same for B
MB = zeros(2,2,m,n);
R = rotation_matrix(2,beta0);
MB(:,:,1,1) = R * b(:,1) * b(:,1)' * R';
MB(:,:,2,1) = R * b(:,2) * b(:,2)' * R';
R = rotation_matrix(2,beta);
for i = 2:n
    MB(:,:,1,i) = R * MB(:,:,1,i-1) * R';
    MB(:,:,2,i) = R * MB(:,:,2,i-1) * R';
end

% compute the quantum behaviour
p = q2p(rho,MA,MB,n,m);

% clean up
clear('MA','MB','i');

%display p in readable form
disp('The behaviour:');
disp(reshape(p,n*m,n*m));

%% standard measurement basis + rotation

MA = zeros(2,2,m,n);

alpha = 1e-3;
R = [ cos(alpha), -sin(alpha);
      sin(alpha), cos(alpha)];
  
  
MA(:,:,1,1) = [1;0] * [1;0]'; % standard basis
MA(:,:,2,1) = [0;1] * [0;1]'; % standard basis

% MA(:,:,1,1) = R * [1;0] * [1;0]' * R'; % rotate standard basis
% MA(:,:,2,1) = R * [0;1] * [0;1]' * R'; %

for i = 2:n
    MA(:,:,1,i) = R * MA(:,:,1,i-1) * R';
    MA(:,:,2,i) = R * MA(:,:,2,i-1) * R';
end

MB(:,:,1,1) = [1;0] * [1;0]'; % standard basis
MB(:,:,2,1) = [0;1] * [0;1]'; % standard basis

for i = 2:n
    MB(:,:,1,i) = R * MB(:,:,1,i-1) * R';
    MB(:,:,2,i) = R * MB(:,:,2,i-1) * R';
end
clear('i');
p = q2p(rho,MA,MB,n,m);
% disp('The behaviour to be checked:');
% disp(reshape(p,n*m,n*m));

%% angle around standard measurement

MA = zeros(2,2,m,n);

alpha = 1e-1;
R = [ cos(alpha), -sin(alpha);
      sin(alpha), cos(alpha)];

k = floor(n/2);
MA(:,:,1,1) = (R'^k) * [1;0] * [1;0]' * (R'^k)'; % standard basis
MA(:,:,2,1) = (R'^k) * [0;1] * [0;1]' * (R'^k)'; % standard basis

for i = 2:n
    MA(:,:,1,i) = R * MA(:,:,1,i-1) * R';
    MA(:,:,2,i) = R * MA(:,:,2,i-1) * R';
end

MB(:,:,1,1) = (R'^k) * [1;0] * [1;0]' * (R'^k)'; % standard basis
MB(:,:,2,1) = (R'^k) * [0;1] * [0;1]' * (R'^k)'; % standard basis

for i = 2:n
    MB(:,:,1,i) = R * MB(:,:,1,i-1) * R';
    MB(:,:,2,i) = R * MB(:,:,2,i-1) * R';
end
clear('i');
p = q2p(rho,MA,MB,n,m);


%% rotation in 3d around the space diagonal

alpha_a = ((1/3) * 2 *  pi)/ (n);
alpha_b = ((1/3) * 2 *  pi)/ (2*n);

% standard basis
b = eye(3);
b_r = b;

% space diagonal
sd = [1;1;1];

MA = zeros(3,3,3,n);

for i = 1:n
    
%     b_r
    b_r = q_rotns(b_r,sd,alpha_a);    

    MA(:,:,1,i) = b_r(:,1) * b_r(:,1)';
    MA(:,:,2,i) = b_r(:,2) * b_r(:,2)';
    MA(:,:,3,i) = b_r(:,3) * b_r(:,3)';
    

end

% same for B

% standard basis
b = eye(3);
b_r = b;

MB = zeros(3,3,3,n);
for i = 1:n

    MB(:,:,1,i) = b_r(:,1) * b_r(:,1)';
    MB(:,:,2,i) = b_r(:,2) * b_r(:,2)';
    MB(:,:,3,i) = b_r(:,3) * b_r(:,3)';
    
    b_r = q_rotns(b_r,sd,alpha_b);    
end


% compute the quantum behaviour
p = q2p(rho,MA,MB,n,m);

% clean up
clear('MA','MB','i');

%display p in readable form
disp('The behaviour:');
disp(reshape(p,n*m,n*m));

